
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;

	if(*idum<=0 || !iy){			//initialize
		if(-(*idum)<1) *idum=1;		//be sure to prevent idum=0.
		else *idum=-(*idum);
		for( j=NTAB+7; j>=0; j--){	//load the shuffle table(after 8 warm-ups)
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if(*idum<0) *idum+=IM;
			if(j<NTAB) iv[j]=*idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if(*idum<0) *idum+=IM;
	j=iy/NDIV;
	iy=iv[j];
	iv[j]=*idum;
	if((temp=AM*iy)>RNMX) return RNMX;
	else return temp;
}






// This is the new version of the EndNormal.cpp  //
// By Shuangge Ma, 11/18/2001, Sunday, 7:30PM

// Change 1: the definition of the EndNormal::EndNormal
//           We take the symmetric case of the minimal detectable effects.

#include "Normal.h"

void mySort(double *p, int size);
double minm(double, double);
double maxm(double, double);
int maxInt(int, int);
int minInt(int, int);
double Sample_Quan(int, double*, double);

EndNormal::EndNormal(){
	order=1;                 // The first or the second endpoint
	num_look=0;				 // number of look
	type = "normal";         // data type: Normal or Survival
	num_simul=0;             // total number of simulations. look_simul=num_simul/num_look
	
	rule_decision = 2;

	look_simul=0;

	mean_Null=0.0;
	mean_Alter=0.0;
	var_Null=0.0;
	var_Alter=0.0;			 // Mean and variance under Null and Alternative hypothesis
	
	event_look=new int[LIMIT_N_LOOK];// The number of events that are expected to happen before each look. 
	inform_time=new double[LIMIT_N_LOOK];   // information spending.

	overall_alpha=0.0;
	overall_beta=0.0;
	spending_num=0;
	//spending is interger taype variable. means which spending
	//function the clinet choose.


	alpha=new double[LIMIT_N_LOOK];
	beta=new double[LIMIT_N_LOOK];

	upper_bound=new double[LIMIT_N_LOOK];
	lower_bound=new double[LIMIT_N_LOOK];
	
	sample_size=0;  // minimal sample size needed for this endpoint
	
	corr_ph=0.0;    // The correlation between the first and the second endpoint. 

	Bi_Alpha = 0;
	Bi_Beta = 0;

	boundary=new double[2*LIMIT_N_LOOK];
	
	ctrl_ind = new int[8];
}



// The folowing is the calculation for each endpoint seperately.
// In the order of calculation.
// Basically, for each endpoint, we run the following functions one by one.

void EndNormal::makeBoundary()
{
	for(int i=0; i<num_look; i++){
		boundary[2*i] = lower_bound[i];
		boundary[2*i+1] = upper_bound[i];
	}

}




//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//@@  The First PART: Ancillary Finctions    @@@@
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

////////  The Initial sample size estimation for Normal type Endpoint:
// Define somewhere QuantilSim=10000  // Number of simulation for calculation of Normal Score	

int EndNormal::one_stage_normal()
{
  int size1;
  double zalpha1, zbeta1;
  double sigma_Null, mu_Null;
  double sigma_Alter, mu_Alter;

  zalpha1 = NormalScoreA(overall_alpha);
  zbeta1 = NormalScoreA(overall_beta);
    
  sigma_Null = sqrt(var_Null);
  sigma_Alter= sqrt(var_Alter);

  mu_Null=mean_Null;
  mu_Alter=mean_Alter;

  size1=(int)pow((zalpha1*sigma_Null+zbeta1*sigma_Alter)/(mean_Alter-mean_Null),2);

  sample_size=size1;

  return size1;  
}

void EndNormal::information()
{
	for(int i=0; i<num_look; i++){
		inform_time[i]=(double)event_look[i]/event_look[num_look-1];
	}
}


void EndNormal::getLookSimul()
{
	if (num_simul!=0)
//		look_simul = (int) num_simul/num_look;
		look_simul = num_simul;

}

// The Information spending on each look should have been calculated by now.
// We calculate the Covariance Matrix of the Normal Endpoint.

// 3.1 The Coviance Matrix of the Normal Type Sequencial Test
double *EndNormal::corr_one_normal()
{
	double tempa, tempb;
	double *information1;
	double *corrm;
	int k, l;

	information1=new double[num_look];
	corrm=new double[num_look*num_look];

	information1=inform_time;

	for(k=0; k < num_look; k++){ 
		for(l=0 ;l < num_look; l++){ 
	  tempa=minm(information1[k], information1[l]);
	  tempb=maxm(information1[k], information1[l]);
	  corrm[k*num_look+l]=sqrt(tempa/tempb);
		}
	}	
    
  return corrm;
}





//      The generation of Null AND Alternative              //
//      Return value will be a matrxi with the following structure:
/*							look1            look2             look3 ........
patient		1			  Null,  Alter    Null, Alter     Null, Alter
   ...
   ...
patient simul_look        Null,  Alter    Null, Alter     Null, Alter

  */


// In the NullAlterGenerator1, we only consider the scale parameter. 
// Since the scale parameter will remain unchanged for different sample size.
// We will consider the location parameter in the NullAlterGenerator2.







double *EndNormal::NullAlterGenerator1()
// trysize1: the sample size n
// muNullAlter (1*2 array): (mean Null, mean Alter)
{
	int loopcount=0;
	int i, j, k, m, n,l;

	int numoflook=num_look;
	int numofsimul=look_simul;

	double *tt;
	tt=new double[num_look];
	for(i=0; i<num_look; i++){
		tt[i]=inform_time[i];
	}

	double ph=corr_ph;     // The correlation between two endpoints.
	double *work1;          // The work Normal matrix
	double *xnew;          // The new covariance matrix
	double *v3sq;          // Square root of xnew;

	work1=new double[2*num_look*look_simul];
	xnew=new double[2*num_look*2*num_look];
	v3sq=new double[2*num_look*2*num_look];

	// Initialize xnew:
	for(i=0; i<num_look; i++) {
		for(j=0; j<num_look; j++) {
			xnew[2*i*(2*num_look)+2*j]=sqrt( tt[minInt(i,j)]/tt[maxInt(i,j)] );
			xnew[2*i*(2*num_look)+2*j+1]=ph*sqrt( tt[minInt(i,j)]/tt[maxInt(i,j)] );
			xnew[(2*i+1)*(2*num_look)+2*j]=ph*sqrt( tt[minInt(i,j)]/tt[maxInt(i,j)] );
			xnew[(2*i+1)*(2*num_look)+2*j+1]=sqrt( tt[minInt(i,j)]/tt[maxInt(i,j)] );
		}
	}

	v3sq=sqrt_matrix(2*num_look, xnew);

	double *transpose;
	transpose=new double[2*num_look*2*num_look];
	for(i=0; i<2*num_look; i++){
		for(j=0; j<2*num_look; j++) {
			transpose[i*2*num_look+j]=v3sq[j*2*num_look+i];
		}
	}

	double* v3sq_sqed=new double[2*num_look*2*num_look];
	for(i=0; i<2*num_look*2*num_look; i++)
		v3sq_sqed[i]=0;

	double temp_ij = 0;

	for(i=0; i<2*num_look; i++){
		for(j=0; j<2*num_look; j++) {
			for(k=0; k<2*num_look; k++){
				v3sq_sqed[i*2*num_look+j]=v3sq_sqed[i*2*num_look+j]+transpose[i*2*num_look+k]*v3sq[k*2*num_look+j];
			}
		}
	}


	work1=Stand_Normal(2*num_look*look_simul); // Initialize Standard Normal

	// The first check procedure:
	double *checkwork;
	int checkii, checkjj;
	int con4;
	double checkmean=0, checkvar=0;

	checkwork=new double[look_simul];

	
	for(checkii=0; checkii<2*num_look; checkii++) {
		checkmean=0;
		checkvar=0.0;
		for(checkjj=0; checkjj<look_simul; checkjj++) {
			checkwork[checkjj]=work1[checkjj*2*num_look+checkii];
			checkmean=checkmean+checkwork[checkjj];
		}

		checkmean=checkmean/look_simul;
		for(checkjj=0; checkjj<look_simul; checkjj++){
			checkwork[checkjj]=checkwork[checkjj]-checkmean;
			checkvar=checkvar+checkwork[checkjj]*checkwork[checkjj];
		}
		checkvar=checkvar/look_simul;
	}





	// Transform accoring to covariance structure:
	double *temp_work2=new double[2*num_look*look_simul];
	double *worktranspose;
	worktranspose=new double[2*num_look*look_simul];

	for (i=0;i<2*num_look*look_simul; i++){
		temp_work2[i]=work1[i];
		worktranspose[i]=0.0;
	}


	for(i=0; i<2*num_look; i++) {
		for(j=0; j<look_simul; j++) {
			for(k=0; k<2*num_look; k++) {
				worktranspose[i*look_simul+j]=worktranspose[i*look_simul+j]+transpose[i*2*num_look+k]*temp_work2[k*look_simul+j];
			}
		}
	}

	for(i=0; i<look_simul; i++){
		for(j=0; j<2*num_look; j++) {
			temp_work2[i*2*num_look+j]=worktranspose[j*look_simul+i];
		}
	}

	// The second check procedure: the same as others.

	double *meanarray, *vararray;
	meanarray=new double[2*num_look];
	vararray=new double[2*num_look];
	checkmean=0, checkvar=0;
	


	checkwork=new double[look_simul];
	
	for(checkii=0; checkii<2*num_look; checkii++) {
		checkmean=0;
		checkvar=0.0;
		for(checkjj=0; checkjj<look_simul; checkjj++) {
			checkwork[checkjj]=temp_work2[checkjj*2*num_look+checkii];
			checkmean=checkmean+checkwork[checkjj];
		}
		checkmean=checkmean/look_simul;
		meanarray[checkii]=checkmean;
		for(checkjj=0; checkjj<look_simul; checkjj++){
			checkwork[checkjj]=checkwork[checkjj]-checkmean;
			checkvar=checkvar+checkwork[checkjj]*checkwork[checkjj];
		}
		checkvar=checkvar/look_simul;
		vararray[checkii]=checkvar;
	}

	//now we check the covariance procedure:

	double * check1, *check2;
	double *covariance;
	double corr;
	int checkkk;
	covariance=new double[2*2*num_look*num_look];
	check1=new double[look_simul];
	check2=new double[look_simul];

	for(checkii=0; checkii<2*num_look*2*num_look; checkii++)
		covariance[checkii]=0;

	for(checkii=0; checkii<2*num_look; checkii++) {
		for(checkjj=checkii; checkjj<2*num_look; checkjj++) {
			corr=0;
			for(checkkk=0; checkkk<look_simul; checkkk++) {
				check1[checkkk]=temp_work2[checkkk*2*num_look+checkii]-meanarray[checkii];
				check2[checkkk]=temp_work2[checkkk*2*num_look+checkjj]-meanarray[checkjj];
			}
			for(checkkk=0; checkkk<look_simul; checkkk++)
				corr=corr+check1[checkkk]*check2[checkkk];
			corr=corr/look_simul;
			corr=corr/sqrt(vararray[checkii]*vararray[checkjj]);
			covariance[checkii*2*num_look+checkjj]=corr;
		}
	}

	return temp_work2;
}






// Now consider the location parameter.

double *EndNormal::NullAlterGenerator2(int trysize2, double *muNullAlter2, double *work1)
{
	int i,j;
	double *work2;
	work2=new double[2*num_look*look_simul];

	for(i=0; i<2*num_look*look_simul; i++)
		work2[i]=work1[i];

	double *sizetemp;
	sizetemp=new double[num_look];

	double *tt;
	tt=new double[num_look];
	tt=inform_time;        // From the Class EndNormal
	
	for(i=0; i<num_look; i++) sizetemp[i]=tt[i]*trysize2;
	// Next add the location parameter.
	
	for(i=0; i<look_simul; i++) {
		for(j=0; j<num_look; j++) {
			work2[i*(2*num_look)+2*j+0]=work1[i*(2*num_look)+2*j+0]+sqrt( (double) sizetemp[j])*muNullAlter2[0];
			work2[i*(2*num_look)+2*j+1]=work1[i*(2*num_look)+2*j+1]+sqrt( (double) sizetemp[j])*muNullAlter2[1];
		}
	}

	return work2;
// Null and Alternative in the same matrix. matrix size: look_simul*(2*num_look)
}



// Given the sample size and The mean of the Alternative, we calculate the Symmetric Boundaries
// The code also known as " THE BIG LOOP"
// Since the generation of Null and Alternative is NOT done here, we do not need the information structure here.


double *EndNormal::Symmetric_Boundary1(int trySize2, double meanAlter1, double *workMatrix)
// We have information externally.
// the workMatrix is from NullAlterGenerator1.
// We only need to seperate Null and Alter from the workMatrix here.
{
	double *boundary1; // The return value of the boudaries.
	double *upperbound, *lowerbound; // The lower and upper bounds
	int i, j, k, l, m, n, o, p, q,r, s;
	int *countAfterNull; // the number of samples left after each look.
	int *countAfterAlter;

	boundary1=new double[num_look*2];
	upperbound=new double[num_look];
	lowerbound=new double[num_look];
	countAfterNull=new int[num_look];
	countAfterAlter=new int[num_look];
	countAfterNull[0]=look_simul;
	countAfterAlter[0]=look_simul;

	double *p0, *p1;
	p0=new double[num_look];
	p1=new double[num_look];

	for(i=0; i<num_look; i++)
		{
			p0[i]=1.0;
			p1[i]=1.0;
		}

	double *alpha3, *beta3;    // The target alpha and beta for each look
	alpha3=new double[num_look];
	beta3=new double[num_look];
	if(spending_num==1){
		alpha3=alpha_beta1(overall_alpha);
		beta3=alpha_beta1(overall_beta);
		alpha=alpha_beta1(overall_alpha);
		beta=alpha_beta1(overall_beta);
	}
		
	if(spending_num==2){
		alpha3=alpha_beta2(overall_alpha);
		beta3=alpha_beta2(overall_beta);
		alpha=alpha_beta2(overall_alpha);
		beta=alpha_beta2(overall_beta);
	}  
	
	if(spending_num==3){
		alpha3=alpha_beta3(overall_alpha);
		beta3=alpha_beta3(overall_beta);
		alpha=alpha_beta3(overall_alpha);
		beta=alpha_beta3(overall_beta);
	}  

	
	// Initialize the alpha and beta spending for each look

	double *workNullAlter;         // The Null and Alternative together
	double *workNull, *workAlter;  // The Null and Alternative seperately
	workNullAlter=new double[2*num_look*look_simul];
	workNull=new double[num_look*look_simul];
	workAlter=new double[num_look*look_simul];

	double *meanArray;
	meanArray=new double[2];

	meanArray[0]=0.0;          // The mean of the Null
	meanArray[1]=meanAlter1;   // The mean of the Alternative

	workNullAlter=NullAlterGenerator2(trySize2, meanArray, workMatrix);

	// The check of mean, variance and covariance of the: workNullAlter


	double *checkwork;
	double *meanarray, *vararray;
	int checkii, checkjj, checkkk;
	double checkmean=0, checkvar=0;
	double corr;
	double *check1, *check2;


	checkwork=new double[look_simul];
	meanarray=new double[2*num_look];
	vararray=new double[2*num_look];
	check1=new double[look_simul];
	check2=new double[look_simul];

	for(checkii=0; checkii<2*num_look; checkii++) {
		checkmean=0.0;
		checkvar=0.0;
		for(checkjj=0; checkjj<look_simul; checkjj++) {
			checkwork[checkjj]=workNullAlter[checkjj*2*num_look+checkii];
			checkmean=checkmean+checkwork[checkjj];
		}
		checkmean=checkmean/look_simul;
		meanarray[checkii]=checkmean;
		for(checkjj=0; checkjj<look_simul; checkjj++){
			checkwork[checkjj]=checkwork[checkjj]-checkmean;
			checkvar=checkvar+checkwork[checkjj]*checkwork[checkjj];
		}
		checkvar=checkvar/look_simul;
		vararray[checkii]=checkvar;
	}


	double *covariance;
	covariance=new double[2*num_look*2*num_look];
	
	for(checkii=0; checkii<2*num_look*2*num_look; checkii++)
		covariance[checkii]=0;

	for(checkii=0; checkii<2*num_look; checkii++) {
		for(checkjj=checkii; checkjj<2*num_look; checkjj++) {
			corr=0;
			for(checkkk=0; checkkk<look_simul; checkkk++) {
				check1[checkkk]=workNullAlter[checkkk*2*num_look+checkii]-meanarray[checkii];
				check2[checkkk]=workNullAlter[checkkk*2*num_look+checkjj]-meanarray[checkjj];
			}
			for(checkkk=0; checkkk<look_simul; checkkk++)
				corr=corr+check1[checkkk]*check2[checkkk];
			corr=corr/look_simul;
			corr=corr/sqrt(vararray[checkii]*vararray[checkjj]);
			covariance[checkii*2*num_look+checkjj]=corr;
		}
	}



	// We then seperate Null and Alternative.
	for(i=0; i<look_simul; i++)	{
		for(j=0;j<num_look; j++) {
			workNull[i*num_look+j]=workNullAlter[i*(2*num_look)+2*j];
			workAlter[i*num_look+j]=workNullAlter[i*(2*num_look)+2*j+1];
		}
	}
	// Initialize the Null and Alternative Accomplished.


	// we now check the mean, varaince and covariance structure of the Null and Alter seperately:

	double mean1, var1;
	int ch1, ch2, ch3;
	double *meanA1, *varA1, *corrA1;
	meanA1=new double[num_look];
	varA1=new double[num_look];
	corrA1=new double[num_look*num_look];

	for(ch1=0; ch1<num_look*num_look; ch1++)
		corrA1[ch1]=0.0;

	for(ch1=0; ch1<num_look; ch1++) {
		mean1=0;
		var1=0;
		for(ch2=0; ch2<look_simul; ch2++) {
			mean1=mean1+workNull[ch2*num_look+ch1];
		}
		mean1=mean1/look_simul;
		meanA1[ch1]=mean1;
	}

	for(ch1=0; ch1<num_look; ch1++){
		var1=0;
		for(ch2=0; ch2<look_simul; ch2++) {
			var1=var1+(workNull[ch2*num_look+ch1]-meanA1[ch1])*(workNull[ch2*num_look+ch1]-meanA1[ch1]);
		}
		var1=var1/look_simul;
		varA1[ch1]=var1;
	}

	for(ch1=0; ch1<num_look; ch1++) {
		for(ch2=0; ch2<num_look; ch2++) {
			for(ch3=0; ch3<look_simul; ch3++) {
				corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]+workNull[ch3*num_look+ch1]*workNull[ch3*num_look+ch2];
			}
			corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]/look_simul;
			corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]/sqrt(varA1[ch1]*varA1[ch2]);
		}
	}

	//the same check for Alter
	for(ch1=0; ch1<num_look*num_look; ch1++)
		corrA1[ch1]=0.0;

	for(ch1=0; ch1<num_look; ch1++) {
		mean1=0;
		var1=0;
		for(ch2=0; ch2<look_simul; ch2++) {
			mean1=mean1+workAlter[ch2*num_look+ch1];
		}
		mean1=mean1/look_simul;
		meanA1[ch1]=mean1;
	}

	for(ch1=0; ch1<num_look; ch1++){
		var1=0;
		for(ch2=0; ch2<look_simul; ch2++) {
			var1=var1+(workAlter[ch2*num_look+ch1]-meanA1[ch1])*(workAlter[ch2*num_look+ch1]-meanA1[ch1]);
		}
		var1=var1/look_simul;
		varA1[ch1]=var1;
	}

	for(ch1=0; ch1<num_look; ch1++) {
		for(ch2=0; ch2<num_look; ch2++) {
			for(ch3=0; ch3<look_simul; ch3++) {
				corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]+(workAlter[ch3*num_look+ch1]-meanA1[ch1])*(workAlter[ch3*num_look+ch2]-meanA1[ch2]);
			}
			corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]/look_simul;
			corrA1[ch1*num_look+ch2]=corrA1[ch1*num_look+ch2]/sqrt(varA1[ch1]*varA1[ch2]);
		}
	}

	// Now we calculate the boundary based on the following:

	// alpha3, beta3: alpha and beta spending
	// upperbound, lowerbound: the calculated value
	// workNull and workAlter

	double *currentNull, *currentAlter;
	double *absNull, *absAlter; // The abs of Null and Alter;
	currentNull=new double[look_simul];
	currentAlter=new double[look_simul];
	absNull=new double[num_look*look_simul];
	absAlter=new double[num_look*look_simul];

	double *alpha4, *beta4;
	alpha4=new double[num_look];
	beta4=new double[num_look];

	double *countNull, *countAlter;
	countNull=new double[num_look];
	countAlter=new double[num_look];

	int li, lj, lk, lm, ln, lo;
	int signNull, signAlter;
	int positionNull, positionAlter;

	for(li=0; li<2*num_look; li++)
		boundary1[li]=0.0;

	for(li=0; li<num_look; li++) {
		upperbound[li]=0;
		lowerbound[li]=0;
	}

	for(li=0; li<num_look*look_simul; li++){
		absNull[li]=absd(workNull[li]);
		absAlter[li]=absd(workAlter[li]);
	}

	for(li=0; li<num_look; li++){
		countNull[li]=0;
		countAlter[li]=0;
	}

	for(ln=0; ln<num_look; ln++) {
		alpha4[ln]=alpha3[ln];
		beta4[ln]=beta3[ln];
	}


	// Again, get the bound for look 0 first:

	for(li=0; li<look_simul; li++){
		currentNull[li]=absNull[li*num_look+0];
		currentAlter[li]=absAlter[li*num_look+0];
	}
	
	upperbound[0]=absd(NormalScoreA(1-alpha3[0]));
	lowerbound[0]=Sample_Quan(look_simul, currentAlter, beta3[0]);

	for(li=0; li<look_simul; li++) {
		if( (currentNull[li] < upperbound[0]) && (currentNull[li] > lowerbound[0]) )
			countNull[0]=countNull[0]+1;
		if( (currentAlter[li] < upperbound[0]) && (currentAlter[li] > lowerbound[0]) )
			countAlter[0]=countAlter[0]+1;
	}

	if( upperbound[0] < lowerbound[0]) {
		boundary1[2*num_look-2]=1;
		boundary1[2*num_look-1]=-1;
		return boundary1;
	}
	// the critical boundary for the rest looks:
	for(lj=1; lj<num_look; lj++) {
		
		positionNull=0;
		positionAlter=0;
		//first get currentNull and currentAlter
		for(lk=0; lk<look_simul; lk++) {
			signNull=0;
			signAlter=0;
			for(lm=0; lm<lj; lm++) {
				if( (absNull[lk*num_look+lm] > upperbound[lm] ) || ( absNull[lk*num_look+lm] <lowerbound[lm]) )
					signNull=1;
				if( (absAlter[lk*num_look+lm] > upperbound[lm] ) || ( absAlter[lk*num_look+lm] <lowerbound[lm]) )
					signAlter=1;
			}

			if(signNull==0) {
				currentNull[positionNull]=absNull[lk*num_look+lj];
				positionNull=positionNull+1;
			}	

			if(signAlter==0) {
				currentAlter[positionAlter]=absAlter[lk*num_look+lj];
				positionAlter=positionAlter+1;
			}	
			
		}
			
		

		alpha4[lj]=alpha3[lj]*look_simul/countNull[lj-1];
		beta4[lj]=beta3[lj]*look_simul/countAlter[lj-1];

		upperbound[lj]=Sample_Quan( positionNull, currentNull, 1-alpha4[lj]);
		if(upperbound[lj]>upperbound[lj-1])
			upperbound[lj]=upperbound[lj-1];

		lowerbound[lj]=Sample_Quan( positionAlter, currentAlter, beta4[lj]);
		if(lowerbound[lj]<lowerbound[lj-1])
			lowerbound[lj]=lowerbound[lj-1];


		if(upperbound[lj] < lowerbound[lj]) {
			boundary1[2*num_look-2]=1;
			boundary1[2*num_look-1]=-1;
			return boundary1;
		}

		//now we count: after the current bound, how many left:
		for(lk=0; lk<look_simul; lk++) {
			signNull=0;
			signAlter=0;
			for(lm=0; lm<=lj; lm++) {
				if( (absNull[lk*num_look+lm] > upperbound[lm] ) || ( absNull[lk*num_look+lm] <lowerbound[lm]) )
					signNull=1;
				if( (absAlter[lk*num_look+lm] > upperbound[lm] ) || ( absAlter[lk*num_look+lm] <lowerbound[lm]) )
					signAlter=1;
			}

			if(signNull==0) countNull[lj]=countNull[lj]+1;
			if(signAlter==0) countAlter[lj]=countAlter[lj]+1;
		}
	}

	for(li=0; li<num_look; li++){
		boundary1[li*2]=lowerbound[li];
		boundary1[li*2+1]=upperbound[li];
	}

	return boundary1;

}














// Next we adjust the sample size for the one endpoint sequencial Normal Test.
// We will store the minimal sample size needed and the corresponding critical boundary in the Endpoint class



// The Main function of the EndNormal case.
// 
void EndNormal::SizeAdjust()
{

	sample_size=one_stage_normal();
	int i, j, k, l, m, n;
	double *worktemp;    // The matrix we generate from NullAlterGenerator1.
	worktemp=new double[2*num_look*look_simul];

	double *workboundtemp;			// The work boundaries
	double *upperboundtemp;			// The upper bound
	double *lowerboundtemp;			// The lower bound
	workboundtemp=new double[2*num_look];
	upperboundtemp=new double[num_look];
	lowerboundtemp=new double[num_look];

	worktemp=NullAlterGenerator1();

	double *checkwork;
	int checkii, checkjj;
	double checkmean=0, checkvar=0;

	checkwork=new double[look_simul];
	
	for(checkii=0; checkii<2*num_look; checkii++) {
		checkmean=0;
		for(checkjj=0; checkjj<look_simul; checkjj++) {
			checkwork[checkjj]=worktemp[checkjj*2*num_look+checkii];
			checkmean=checkmean+checkwork[checkjj];
		}
		checkmean=checkmean/look_simul;
		for(checkjj=0; checkjj<look_simul; checkjj++){
			checkwork[checkjj]=checkwork[checkjj]-checkmean;
			checkvar=checkvar+checkwork[checkjj]*checkwork[checkjj];
		}
		checkvar=checkvar/look_simul;
	}


	double lowerdiff, upperdiff, workdiff;
	int lowersize, midsize, uppersize;

	midsize=sample_size;			// From the EndNormal class.  The initial guess of size

	double workAlter;
	workAlter=mean_Alter;

	// Now initialize worktemp:
	workboundtemp=Symmetric_Boundary1(midsize, workAlter, worktemp);

	for(i=0; i<num_look; i++) {
		lowerboundtemp[i]=workboundtemp[2*i];
		upperboundtemp[i]=workboundtemp[2*i+1];
	}

	int signuppersize=0;

	//if we have reached the upper bound of size
	if( (lowerboundtemp[num_look-1]==1) && (upperboundtemp[num_look-1]==-1) )
		signuppersize=1;		

	uppersize=midsize;
	// Search for the upper limit of the sample size needed.

	while( signuppersize==0 ) {

		uppersize= (int) (1.2*uppersize);
		workboundtemp=Symmetric_Boundary1(uppersize, workAlter, worktemp);

		int uppersize_int1;

		for(i=0; i<num_look; i++) {
			lowerboundtemp[i]=workboundtemp[2*i];
			upperboundtemp[i]=workboundtemp[2*i+1];
		}

		if( (lowerboundtemp[num_look-1]==1) && (upperboundtemp[num_look-1]==-1) )
			signuppersize=1;		
	}


	// Now let's find the lower limit of sample size:
	lowersize=midsize;
	int signlowsize=0;

	workboundtemp=Symmetric_Boundary1(lowersize, workAlter, worktemp);
	for(i=0; i<num_look; i++) {
		lowerboundtemp[i]=workboundtemp[2*i];
		upperboundtemp[i]=workboundtemp[2*i+1];
	}

	if( upperboundtemp[num_look-1] > lowerboundtemp[num_look-1] )
		signlowsize=1;

	while( signlowsize==0 ) {
		lowersize= int(lowersize/1.2);

		workboundtemp=Symmetric_Boundary1(lowersize, workAlter, worktemp);
		for(i=0; i<num_look; i++) {
			lowerboundtemp[i]=workboundtemp[2*i];
			upperboundtemp[i]=workboundtemp[2*i+1];
		}
		if( upperboundtemp[num_look-1] > lowerboundtemp[num_look-1] )
			signlowsize=1;
	}

	// Now let's find the minimal sample size:

	midsize=int( (lowersize+uppersize)/2 );

	workboundtemp=Symmetric_Boundary1(midsize, workAlter, worktemp);
	for(i=0; i<num_look; i++) {
		lowerboundtemp[i]=workboundtemp[2*i];
		upperboundtemp[i]=workboundtemp[2*i+1];
	}
	workdiff=upperboundtemp[num_look-1]-lowerboundtemp[num_look-1];

	int counter=0;   // Add a counter to avoid infinite times
	//while upper and lower not equal:
	while( (absd(workdiff)>0.002) && (counter<1000) && ( (uppersize-lowersize) >1 )) {
		
		//case 1: upper > lower, then increase 
		if(workdiff>0) {
			lowersize=midsize;
			midsize=int( (lowersize+uppersize)/2 );
			workboundtemp=Symmetric_Boundary1(midsize, workAlter, worktemp);
			for(i=0; i<num_look; i++) {
				lowerboundtemp[i]=workboundtemp[2*i];
				upperboundtemp[i]=workboundtemp[2*i+1];
			}
			workdiff=upperboundtemp[num_look-1]-lowerboundtemp[num_look-1];
			counter=counter+1;
		}

		//case2: upper<lower, then decrease:
		if(workdiff<0) {
			uppersize=midsize;
			midsize=int( (lowersize+uppersize)/2 );
			workboundtemp=Symmetric_Boundary1(midsize, workAlter, worktemp);
			for(i=0; i<num_look; i++) {
				lowerboundtemp[i]=workboundtemp[2*i];
				upperboundtemp[i]=workboundtemp[2*i+1];
			}
			workdiff=upperboundtemp[num_look-1]-lowerboundtemp[num_look-1];
			counter=counter+1;
		}
	}

	cout<<"The final size calculated as:"<<"\t"<<midsize<<endl;

	// Store the values in the EndNormal Type variable.
	sample_size=midsize;
	lowerboundtemp[num_look-1]=upperboundtemp[num_look-1];
	for(i=0; i<num_look; i++) {
		upper_bound[i]=upperboundtemp[i];
		lower_bound[i]=lowerboundtemp[i];
		boundary[2*i]=lower_bound[i];
		boundary[2*i+1]=upper_bound[i];
	}

/*	cout<<"The boundary is as following"<<endl;
	cout<<"upperbound"<<"\t";
	for(i=0; i<num_look; i++)
		cout<<upperboundtemp[i]<<"\t";
	cout<<endl;
	cout<<"lowerbound"<<"\t";
	for(i=0; i<num_look; i++)
		cout<<lowerboundtemp[i]<<"\t";
*/	cout<<endl<<"End of Size Adjustment and Boundary Calculation."<<endl;

}







// Some more Ancillary Functions:


void mySort(double *p, int size)
{
	int flag=1; // if flag=1, need to do more work
	double tmp;
	int count=0;

	while (flag!=0){
		flag=0;
		for(int i=0; i<size-1-count; i++){
			if(p[i]>p[i+1]){
				tmp=p[i];
				p[i]=p[i+1];
				p[i+1]=tmp;
				flag=1;
			}
		}
		count++;
	}
}





double maxm(double a, double b)
{ 
  if(a>=b) 
	  return a;
  else 
	  return b;
}

double minm(double a, double b)
{
  if(a>=b) 
	  return b;
  else 
	  return a;
}

int maxInt(int a, int b)
{
	return a>b? a : b;
}

int minInt(int a, int b)
{
	return a<b? a : b;
}

double absd(double a)
{
	return a>0? a: -a;
}
/*
void showBoundary(int look)
{
	int factor = 400;
  	glColor3f(1,0,0); 
	glLineWidth(2);
	glBegin(GL_LINE_STRIP);
	for (int i=0; i<=count1_information; i++){
		glVertex3f(factor*i, factor*NormalEnd1->boundary[2*i],0);
	}
	glEnd();

	glColor3f(1,1,0); 
	glLineWidth(2);
	glBegin(GL_LINE_STRIP);
	for (i=0; i<=count1_information; i++){
		glVertex3f(factor*i, factor*NormalEnd1->boundary[2*i+1],0);
	}
	glEnd();

}
*/

double* Stand_Normal(int i)   //yu endpoint wu guan
{
  double *p, *workarray;
  double r, theta;
  double mean=0.0;
  double var=0.0;

  int con1;

  int j;
  int n=0; // the count
  p=new double[i];
  workarray= new double[i];
  int seed1= -117*(int) ( (double) ( rand() % 2000) ) / 2000.0;
  long* idum1=new long(seed1);
  int seed2= -117*(int) ( (double) ( rand() % 2000) ) / 2000.0;
  long* idum2=new long(seed2);

  //WE USE TIME TO CONTROL THE RANDOM NUMBER

  while(n<i){
	workarray[n]=ran1(idum1); 
	workarray[n+1]=ran1(idum2); 

	if(workarray[n]>0){
		r=sqrt(-2*log(workarray[n]));
		theta=2*Pi*workarray[n+1];
		p[n]=r*cos(theta);
		p[n+1]=r*sin(theta);
		n=n+2;
		}  
	} // We here avoid the extremely small case which will make the NA case
/*
  double *check;
  check=new double[2+1*1];
  check=NormalCheck(1, i, workarray);

  for(n=0; n<3; n++)
	  cout<<check[n]<<"\t";
  cout<<endl<<"check of Normal in Stand_Normal"<<endl;
  cin>>n;

*/
  //standardize to guaranttee the mean and variance.
  return p;
}





/*

// Standard Normal Generator:
double* Stand_Normal(int i)   //yu endpoint wu guan
{
  double *p, *workarray;
  double r, theta;
  double mean=0.0;
  double var=0.0;

  int j;
  int n=0; // the count
  p=new double[i];
  workarray= new double[i];

  //WE USE TIME TO CONTROL THE RANDOM NUMBER
  srand ((unsigned)time(NULL));

  while(n<i){
	workarray[n]=( (double) ( rand() % 2000) ) / 2000.0  ; 
	workarray[n+1]=( (double) ( rand() % 2000) ) / 2000.0  ; 
    
	if(workarray[n]>0.00000000001){
		r=sqrt(-2*log(workarray[n]));
		theta=2*Pi*workarray[n+1];
		p[n]=r*cos(theta);
		p[n+1]=r*sin(theta);
		n=n+2;
		}  
	} // We here avoid the extremely small case which will make the NA case

  for(j=0; j<i; j++)
	  mean=mean+p[j];
  mean=mean/i;
  for(j=0; j<i; j++){
	  p[j]=p[j]-mean;
	  var=var+p[j]*p[j];
  }
  var=var/i;
  for(j=0; j<i; j++)
	  p[j]=p[j]/sqrt(var);

  delete [] workarray;

  //standardize to guaranttee the mean and variance.
  return p;
}



*/



//  1.3: The Calculation of Normal Score:

double normalscore(double quantile, int numsimul)  //wuguan
{
  double *worknormal=new double[numsimul];
  int order1;

  worknormal=Stand_Normal(numsimul);

  mySort(worknormal, numsimul); 
  order1= (int) ( quantile*numsimul - 1 );
  double normal_score=-worknormal[order1];

  delete [] worknormal;
  return normal_score;

}



// The square root of a matrix by LP
double *sqrt_matrix(int size, double *A)  // Transpose(B)*(B) = A , B is an upper triangular matrix,
{
	double *B = new double[size*size];
	int i=0, j=0;

	for(i=0; i<size; i++){			//initialize
		for(j=0; j<size; j++){
			B[i*size+j] = 0;
		}
	}
	double temp_Bii2=0, temp_Bij=0;

	int p=0, q=0;
	for(i=0; i<size; i++){
		temp_Bii2 = A[i*size+i];
		for(p=0; p<i; p++){
			temp_Bii2 -= pow(B[p*size+i],2);
		}
		B[i*size+i] = pow(temp_Bii2, 0.5);

		for(j=i+1; j<size; j++){
			temp_Bij = A[i*size+j];
			for(q=0; q<i; q++){
				temp_Bij -= B[q*size+i]*B[q*size+j];
			}
			B[i*size+j]=temp_Bij/B[i*size+i];
		}
	}

	return B;
}



// The new squre root of a matrix. 
double *sqrt_matrixNew(int size, double *A)  // B' * B = A
{
	int i, j, k, l, m, n;
	double *B;
	B=new double[size*size];
	double tempr, tempr2;

	for(i=0; i<size*size; i++)
		B[i]=A[i];
	
	for(i=0; i<size; i++){
		for(j=0; j<size; j++) {
			cout<<setprecision(2)<<A[i*size+j]<<"\t";
		}
		cout<<endl;
	}

	for(i=0; i<size*size; i++)
		B[i]=0.0;

	B[0]=sqrt(A[0]);

	for(j=1; j<size; j++)
		B[0*size+j]=A[0*size+j]/B[0];

	
	for(i=1; i<size; i++){
		tempr=0.0;
		for(l=0; l<=i-1; l++){ 
			tempr=tempr+B[0*size+i]*B[0*size+i];
		}	
	}
      //     cout<<tempr<<endl;
      
	B[i*size+i]=sqrt(A[i*size+i]-tempr);

    for(j=i+1; j<size; j++)	{
		tempr2=0.0;
		for(l=0; l<=i-1; l++) {
			tempr2=tempr2+B[l*size+i]*B[l*size+j];
	    }
	  //  cout<<tempr2<<endl;
	
		B[i*size+j]=(A[i*size+j]-tempr2)/B[i*size+i];
	}


	int con1;
	cin>>con1;
	
	return B;


}




double Sample_Quan(int size, double *array1, double percentage)
{
  double *p;
  int flag=1; // if flag=1, need to do more work
  double tmp, score;
  int count=0, position;
  p=new double[size];  

  p=array1;

  while (flag!=0)
    {
      flag=0;
      for(int i=0; i<size-1-count; i++){
		if(p[i]>p[i+1]){
		tmp=p[i];
		p[i]=p[i+1];
		p[i+1]=tmp;
		flag=1;
		}
      }
      count++;
    }
	
  position=(int) (percentage*(size-1) );

  position=position;

//  if( position <1) position=1;
//  if( position ==size) position=size-1;
	
  score=array1[position];
	
  int i;

//  cout<<"percentage"<<percentage<<endl;
//  cout<<"position"<<"\t"<<position<<endl;
//  cout<<"score"<<"\t"<<score<<endl;
//  cin>>i;

  return score;            // Return value is the percentile
}






// Ancillary Funcation: the Alpha and Beta spending function:

// For spending_num=1;
double* EndNormal::alpha_beta1(double totalAlphaBeta)
{
	double *information3;
	information3=new double[num_look];

	int i;
	for(i=0; i<num_look; i++)
		information3[i]=inform_time[i];

	double *alphareturn;
	double *alphatemp;
	alphareturn=new double[num_look];
	alphatemp=new double[num_look];

	for(i=0; i< num_look; i ++){
		alphareturn[i]=totalAlphaBeta*(1-exp(4*information3[i]))/(1-exp(4));
	}

//	cout<<totalAlphaBeta<<endl;
//    cout<<"information"<<"\t"<<information3[0]<<"\t"<<information3[1]<<endl;
	alphatemp[0]=alphareturn[0];
	for(i=1; i<num_look; i++)
		alphatemp[i]=alphareturn[i]-alphareturn[i-1];

	return alphatemp;
}

// For spending_num=2;

double *EndNormal::alpha_beta2(double totalAlphaBeta)
{
	double *alphareturn;
	double *alphatemp;
	alphareturn=new double[num_look];
	alphatemp=new double[num_look];

	int i;

	for(i=0; i<num_look; i++) {
		alphareturn[i]=totalAlphaBeta*inform_time[i];
	}
	alphatemp[0]=alphareturn[0];
	for(i=1; i<num_look; i++){
		alphatemp[i]=alphareturn[i]-alphareturn[i-1];
	}

	return alphatemp;
}

// For spending_num=3, 4..... (God Save Poor)

double *EndNormal::alpha_beta3(double totalAlphaBeta)
{
	double *alphareturn;
	double *alphatemp;
	alphareturn=new double[num_look];
	alphatemp=new double[num_look];
	int i;

	for(i=0; i< num_look; i++) {
		alphareturn[i]=totalAlphaBeta*inform_time[i]*inform_time[i];
	}
	alphatemp[0]=alphareturn[0];
	for(i=1; i<num_look; i++){
		alphatemp[i]=alphareturn[i]-alphareturn[i-1];
	}

	return alphatemp;

}



double NormalScoreA(double prob)
{
  double e=2.7182818284;
  
  double P0=-0.322232431088;
  double P1=-1.0;
  double P2=-0.342242088547;
  double P3=-0.204231210245e-1;
  double P4=-0.453642210148e-4;

  double Q0=0.993484626060e-1;
  double Q1=0.588581570495;
  double Q2=0.531103462366;
  double Q3=0.103537752850;
  double Q4=0.38560700634e-2;

  if(prob>0.5)
    prob=1-prob;

  double y;
  y=sqrt( log(1/(prob*prob))/log(e));
  
  double quartile;
  quartile=y+((((y*P4+P3)*y+P2)*y+P1)*y+P0)/((((y*Q4+Q3)*y+Q2)*y+Q1)*y+Q0);

  return quartile;

}
